#version 330

#ifdef VERTEX_SHADER

out vec2 vertex_texcoord;

void main(void)
{
    vec3 positions[3]= vec3[3]( vec3(-1, 1, -1), vec3( -1, -3, -1), vec3( 3,  1, -1) );
    gl_Position = vec4(positions[gl_VertexID], 1.0);
    vertex_texcoord = positions[gl_VertexID].xy / 2.0 + 0.5;
}

#endif

#ifdef FRAGMENT_SHADER

in vec2 vertex_texcoord;
out vec4 fragment_color;

uniform sampler2D image;

uniform float compression = 1;
uniform float saturation = 1;
uniform float zoom;
uniform vec4 channels= vec4(1, 1, 1, 1);
uniform float gray= 0;

uniform vec2 center;
uniform int alpha;  
uniform int lod;  
uniform int variance;  

uniform int tile;
uniform int tile_size;
uniform int width;
uniform int height;

uniform int normalized;
uniform float scale;

void main(void)
{
    const vec3 rgby = vec3(0.3, 0.59, 0.11);
    float k1 = 1.0 / pow(saturation, 1.0 / compression); // normalisation : saturation == blanc
    
    // zoom
    vec2 zoom_texcoord= vertex_texcoord;
    if(zoom != 1) zoom_texcoord = (vertex_texcoord - vec2(0.5, 0.5)) / zoom + center;
    
    vec4 color = textureLod(image, zoom_texcoord, lod);

    if( color.w < 0 )
    {
        fragment_color = color;
        return;
    }

    if( normalized == 0 )
    {
        color = vec4(color.rgb * scale,1);
    }

    if(any(isnan(color.xyz)) || any(isinf(color.xyz)))
    {
        fragment_color = vec4(1, 0, 1, 1);
        return;
    }

    // channels
    if(channels != vec4(0, 0, 0, 1))
        color.rgb= channels.rgb * color.rgb;
    else
        color.rgb= color.aaa;
    
    if(gray > 0.0)
    {
        float g= max(color.r, max(color.g, color.b));
        color.rgb= vec3(g, g, g);
    }
    
    // applique la compression (gamma)
    float y = dot(color.rgb, rgby);  // normalisation de la couleur : (color / y) == teinte
    color = (color / y) * k1 * pow(y, 1.0 / compression);
    
    fragment_color= vec4(clamp(color.rgb, vec3(0.0), vec3(1.0)), 1);

    if( tile > 0 )
    {
        ivec2 pixel = ivec2(zoom_texcoord * vec2(width, height));

        int tilex = int(ceil(float(width) / float(tile_size)));
        int tiley = int(ceil(float(height) / float(tile_size)));
        int ymin = (tile / tilex) * tile_size;
        int ymax = min(ymin + tile_size, height);
        int xmin = (tile % tilex) * tile_size;
        int xmax = min(xmin + tile_size, width);


        if( pixel.x >= xmin && pixel.x <= xmax && pixel.y >= ymin && pixel.y <= ymax )
        if( 
            (pixel.x >= xmin && pixel.x <= xmin+3) || 
            (pixel.x <= xmax && pixel.x >= xmax-3) ||
            (pixel.y >= ymin && pixel.y <= ymin+3) || 
            (pixel.y <= ymax && pixel.y >= ymax-3) 
            ) 
        {
            fragment_color = vec4(0,0,0,1);
        }    

        // if( pixel.x >= max(0,xmin-2) && pixel.x <= min(width,xmin+2)  )
        // {
        //     fragment_color = vec4(0,0,0,1);
        // }

    }
}

#endif