#version 330

#ifdef VERTEX_SHADER
out vec2 vertex_texcoord;

void main(void)
{
    vec3 positions[3]= vec3[3]( vec3(-1, 1, -1), vec3( -1, -3, -1), vec3( 3,  1, -1) );
    
    gl_Position= vec4(positions[gl_VertexID], 1.0);
    vertex_texcoord= positions[gl_VertexID].xy / 2.0 + 0.5;
}

#endif

#ifdef FRAGMENT_SHADER
uniform float compression= 1;
uniform float saturation= 1;
uniform vec4 channels= vec4(1, 1, 1, 1);
uniform float gray= 0;
uniform float difference= 0;

uniform int split;
uniform sampler2D image;
uniform sampler2D image_next;

uniform float zoom;
uniform vec2 center;

uniform int colormap;


uniform int width;
uniform int height;

uniform int graph;
uniform vec2 line;

in vec2 vertex_texcoord;
out vec4 fragment_color;

vec3 colordata[33]=vec3[](
    vec3(59.0/255.0,76.0/255.0,192.0/255.0),
    vec3(68.0/255.0,90.0/255.0,204.0/255.0),
    vec3(77.0/255.0,104.0/255.0,215.0/255.0),
    vec3(87.0/255.0,117.0/255.0,225.0/255.0),
    vec3(98.0/255.0,130.0/255.0,234.0/255.0),
    vec3(108.0/255.0,142.0/255.0,241.0/255.0),
    vec3(119.0/255.0,154.0/255.0,247.0/255.0),
    vec3(130.0/255.0,165.0/255.0,251.0/255.0),
    vec3(141.0/255.0,176.0/255.0,254.0/255.0),
    vec3(152.0/255.0,185.0/255.0,255.0/255.0),
    vec3(163.0/255.0,194.0/255.0,255.0/255.0),
    vec3(174.0/255.0,201.0/255.0,253.0/255.0),
    vec3(184.0/255.0,208.0/255.0,249.0/255.0),
    vec3(194.0/255.0,213.0/255.0,244.0/255.0),
    vec3(204.0/255.0,217.0/255.0,238.0/255.0),
    vec3(213.0/255.0,219.0/255.0,230.0/255.0),
    vec3(221.0/255.0,221.0/255.0,221.0/255.0),
    vec3(229.0/255.0,216.0/255.0,209.0/255.0),
    vec3(236.0/255.0,211.0/255.0,197.0/255.0),
    vec3(241.0/255.0,204.0/255.0,185.0/255.0),
    vec3(245.0/255.0,196.0/255.0,173.0/255.0),
    vec3(247.0/255.0,187.0/255.0,160.0/255.0),
    vec3(247.0/255.0,177.0/255.0,148.0/255.0),
    vec3(247.0/255.0,166.0/255.0,135.0/255.0),
    vec3(244.0/255.0,154.0/255.0,123.0/255.0),
    vec3(241.0/255.0,141.0/255.0,111.0/255.0),
    vec3(236.0/255.0,127.0/255.0,99.0/255.0),
    vec3(229.0/255.0,112.0/255.0,88.0/255.0),
    vec3(222.0/255.0,96.0/255.0,77.0/255.0),
    vec3(213.0/255.0,80.0/255.0,66.0/255.0),
    vec3(203.0/255.0,62.0/255.0,56.0/255.0),
    vec3(192.0/255.0,40.0/255.0,47.0/255.0),
    vec3(180.0/255.0,4.0/255.0,38.0/255.0)
);

void main(void)
{
    const vec3 rgby= vec3(0.3, 0.59, 0.11);
    float k1= 1.0 / pow(saturation, 1.0 / compression); // normalisation : saturation == blanc
    
    // zoom
    vec2 zoom_texcoord= vertex_texcoord;
    if(zoom != 1)
        zoom_texcoord= (vertex_texcoord - center) / zoom + center;
    
    // split
    vec4 color= texture(image, zoom_texcoord);
    vec4 color_next= texture(image_next, zoom_texcoord);
    
    if(gl_FragCoord.x >= split)
    {
        if(difference == 0)
            color= color_next;
        
    }

    if(difference == 1)
    {
        color= abs(color_next - color);
        float norm = length(color.xyz); 
        color = vec4(norm,norm,norm,1);
    }

    // nan
    if(any(isnan(color)) || any(isinf(color)))
    {
        fragment_color= vec4(1, 0, 1, 1);
        return;
    }
    
    // graph
    vec4 gcolor= texture(image, vec2(zoom_texcoord.x, line.x));
    vec4 gcolor_next= texture(image, vec2(zoom_texcoord.x + dFdx(zoom_texcoord.x), line.x));
    
    if(gl_FragCoord.x >= split)
    {
        if(difference == 0)
        {
            gcolor= texture(image_next, vec2(zoom_texcoord.x, line.x));
            gcolor_next= texture(image_next, vec2(zoom_texcoord.x + dFdx(zoom_texcoord.x), line.x));
        }
        else
        {
            gcolor= abs(texture(image_next, vec2(zoom_texcoord.x, line.x)) - gcolor);
            float norm = length(gcolor.xyz); 
            gcolor = vec4(norm,norm,norm,1);
            gcolor_next= abs(texture(image_next, vec2(zoom_texcoord.x + dFdx(zoom_texcoord.x), line.x)) - gcolor_next);
            norm = length(gcolor_next.xyz);
            gcolor_next = vec4(norm,norm,norm,1);
        }
    }
    
    // channels
    // applique la compression (gamma)
    if(channels != vec4(0, 0, 0, 1))
        color.rgb= channels.rgb * color.rgb;
    else
        // visualisation du canal alpha seul
        color.rgb= color.aaa;
    
    if(gray > 0.0)
    {
        float g= max(color.r, max(color.g, color.b));
        color.rgb= vec3(g, g, g);
    }
    
    float y= dot(color.rgb, rgby);  // normalisation de la couleur : (color / y) == teinte
    color= (color / y) * k1 * pow(y, 1.0 / compression);
    
    // graph
    if(graph > 0)
    {
        float gy= dot(gcolor.rgb, rgby);
        gcolor= (gcolor / gy) * k1 * pow(gy, 1.0 / compression);
        float gyn= dot(gcolor_next.rgb, rgby);
        gcolor_next= (gcolor_next / gyn) * k1 * pow(gyn, 1.0 / compression);
        
        // intervalle
        vec4 ymin= min(gcolor, gcolor_next) * 0.99;
        vec4 ymax= max(gcolor, gcolor_next);
        
        // intersection intervalle
        vec4 gline= vec4(0, 0, 0, 0);
        if((channels.r > 0) && (vertex_texcoord.y >= ymin.r) && (vertex_texcoord.y <= ymax.r))
            gline.r= 1.0;
        if((channels.g > 0) && (vertex_texcoord.y >= ymin.g) && (vertex_texcoord.y <= ymax.g))
            gline.g= 1.0;
        if((channels.b > 0) && (vertex_texcoord.y >= ymin.b) && (vertex_texcoord.y <= ymax.b))
            gline.b= 1.0;

        if(gline.r > 0 || gline.g > 0 || gline.b > 0)
            color= gline;
            
        // draw graph
        if(abs(gl_FragCoord.y - line.y) < 1)
            color= vec4(0.8, 0.5, 0, 1);
    }

    // draw split
    if(abs(gl_FragCoord.x - split) < 1)
        color= vec4(0.8, 0.5, 0, 1);
    
    fragment_color= vec4(clamp(color.rgb, vec3(0.0), vec3(1.0)), 1.0);

    if( colormap > 0 )
    {
        ivec2 pixel = ivec2(vertex_texcoord * vec2(width, height));

        int map_width = width / 20;
        int offset = 40; 
               
        int xmin = width - (offset + map_width);
        int ymin = offset;
        int xmax = width - offset;
        int ymax = height - offset;

        // border
        // color scale
        if( pixel.x >= xmin && pixel.x <= xmax && pixel.y >= ymin && pixel.y <= ymax )
        {
            if( pixel.x == xmin || pixel.x == xmax || pixel.y == ymin || pixel.y == ymax )
            {
                fragment_color = vec4(0,0,0,1);
            }    
            else
            {
                float value = 32.0 * float(pixel.y - ymin) / float(ymax - ymin);
                int imin = int(max(0.0,floor(value)));
                int imax = int(min(32.0,ceil(value)));
                float frac = fract(value);
                fragment_color = mix(vec4(colordata[imin],1),vec4(colordata[imax],1),frac);
            }
        }
        // image
        else
        {   
            float value = 32.0 * max(fragment_color.r, max(fragment_color.g, fragment_color.b));
            int imin = int(max(0.0,floor(value)));
            int imax = int(min(32.0,ceil(value)));
            float frac = fract(value);
            fragment_color = mix(vec4(colordata[imin],1),vec4(colordata[imax],1),frac);
        }
    }
}

#endif