solution "minimvpt"
	configurations { "release", "debug", "profile", "single", "debugnopt" }

	includedirs { ".", "src/gKit" }

	configuration "linux"
		buildoptions { "-mtune=native -march=native -std=c++17 -W -Wall -Wextra -Wsign-compare -Wno-unused-parameter -Wno-unused-function -Wno-unused-variable", "-pipe" }
		links { "GLEW", "SDL2", "SDL2_image", "GL" }
		buildoptions { "-fopenmp"}
		linkoptions { "-fopenmp"}

	configuration { "release" }
		defines { "GK_RELEASE" }
		defines { "NDEBUG" }
		flags { "OptimizeSpeed" }
		buildoptions{ "-O3" }
		linkoptions{ "-O3" }
		buildoptions { "-flto"}
		linkoptions { "-flto"}

	configuration { "debug" }
		defines { "DEBUG" }
		flags { "Symbols" }
		buildoptions { "-Og"}
		linkoptions { "-Og"}

	configuration { "profile" }
		defines { "PROFILE" }
		flags { "Symbols" }
		buildoptions { "-pg -Og"}
		linkoptions { "-pg -Og"}
		buildoptions { "-flto"}
		linkoptions { "-flto"}

	configuration { "single" }
		defines { "SINGLE_THREAD" }
		flags { "Symbols" }
		buildoptions { "-g"}
		linkoptions { "-g"}

	configuration { "debugnopt" }
		defines { "DEBUG" }
		flags { "Symbols" }
	
-- description of common files
gkit_files = { "src/gKit/*.cpp", "src/gKit/*.h" }
noise_files = { "extern/FastNoise/*.cpp", "extern/FastNoise/*.h" }
nanovdb_files = { "extern/openvdb/nanovdb/nanovdb/NanoVDB.h" }
nanovdb = "extern/openvdb/nanovdb"

project("renderer")
		language "C++"
		kind "ConsoleApp"
		targetdir "bin"
		
		-- embree
		links { "embree3", "tbb" }
		defines { "EMBREE" }

		-- fast noise
		includedirs { "extern/FastNoise" }

		-- NanoVDB
		includedirs { nanovdb }
		links { "z" }

		-- renderer files
		files { gkit_files, noise_files, nanovdb_files, "renderer/*.cpp", "renderer/*.h" }
		files { "renderer/integrators/*.cpp", "renderer/integrators/*.h" }
		files { "renderer/textures/*.cpp", "renderer/textures/*.h" }