# Mini MVPT

Minimalist sample code of the paper "Volumetric Multi-View Rendering" EG2022.

## Build

This code has been tested on linux only and requires:
- premake4 to build,
- and OpenGL, GLEW, SDL2, SLD2_image, Intel TBB, Intel Embree 3.X and Zlib to run,
which can be installed with the following command:
`sudo apt-get install premake4 libsdl2-dev libsdl2-image-dev libglew-dev libtbb-dev libembree-dev` 

To build the code:
`premake4 gmake`
`make renderer -j16`

## Usage

`bin/renderer <scene_file.scn>`

## Example scenes

We included 5 scenes extracted from the paper:
- from Fig. 10: `data/ArticleCloubes`
- from Fig. 4: `data/ArticleDisney`
- from Fig. 4: `data/ArticleDragonLightfield`
- from Fig. 5: `data/ArticleDragonAnisotropic`
- from Fig. 5: `data/ArticleDragons3`

Each directory contains the scene files of various algorithms as described in the article:
- mvpt: multi-view path tracing with the MWIS estimator for combining base and reuse samples
- mismvpt: multi-view path tracing with the MIS estimator for combining base and reuse samples
- amvpt: multi-view path tracing with the MWIS estimator and adaptive sampling
- vpt: classic independent path tracing

## Structure

- `src/` contains a part of the gKit math library (by jean-claude iehl)
- `renderer/` contains the unpolished renderer source code.
- `renderer/integrators/` contains the integrators used in the paper
- `renderer/media/media.h` contains the implementation of volume shift mappings

## Scene files

The scene description is currently hacky and undocumented, but some useful parameters are:
- `viewer` set to 0 renders with no gui, set to 1 with OpenGL preview
- `save` set to 0 no result savings, set to 1 with saving
- `time_limit` or `sample_limit` set the desired time or sample count to reach 
- `partial_times` or `partial_samples` set the desired times or sample counts to save images 
