#include "normalintegrator.h"

NormalIntegrator::NormalIntegrator( 
    std::vector<Camera>&      aCameras, 
    Builder&                  aBuilder, 
    const IntegratorConfig&   aConfig 
) : AbstractIntegrator(aCameras, aBuilder, aConfig) {}

void NormalIntegrator::Render()
{
    int mNumThreads = omp_get_max_threads();
    std::vector<Sampler> mSamplers(mNumThreads);

    // while integrator is not notified to stop
    mTimer.reset();
    while( !mStop )
    {
        // loop over cameras
        for( int CamID = 0, CameraNumber = cameras.size(); CamID < CameraNumber; ++CamID )
        {
            int width = cameras[CamID].width;
            int height = cameras[CamID].height;
            Camera& camera = cameras[CamID];

            #pragma omp parallel
            {      
                // sampler selection
                Sampler& sampler = mSamplers[omp_get_thread_num()]; 

                // render pass 
                #pragma omp for schedule(dynamic, 8)
                for( int px = 0; px < width; ++px )
                for( int py = 0; py < height; ++py )
                {
                    CameraSample cs = camera.sample(sampler, px, py, true);
                    Ray& ray = cs.ray;
                    vec2& pixel = cs.filmPosition;

                    Color color = Black();
                    if( mBuilder.primary( ray ) )
                    {   
                        Interpolation interpolation = mBuilder.interpolate( ray );
                        Vector hitn = interpolation.n;
                        color = Color(std::abs(hitn.x), std::abs(hitn.y), std::abs(hitn.z));
                    }
                    camera.film.splat(pixel, color);
                }
            }
        }
    }
}
