#ifndef SCENE_H
#define SCENE_H

#include "bezier.h"

#include "renderer/integrators/abstractintegrator.h"
#include "renderer/camera.h"

// scene file description

// newcameras
    // # required
    // file(s)
    // # optionnal
    // lerp 10

// newgeometry
    // # required
    // file(s)
    // # optionnal
    // compact 0

bool use_camera(const int id, const std::vector<int>& only)
{
    return only.empty() || std::find(only.cbegin(), only.cend(), id) != only.cend();
}
void interpolate_bezier(std::vector<Camera>& cameras, int viewers, const std::vector<int>& only)
{
    if( cameras.size() <= 3 ) return;
    int w = cameras[0].width, h = cameras[0].height;
    float radius = cameras[0].lens.radius();

    float fov = cameras[0].fov;
    for(auto& camera : cameras)
    {
        printf("%f %f %d %d\n", camera.width, camera.height, w, h);
        assert(camera.width == w && camera.height == h);
    }

    // interpolate cameras if needed
    if( viewers )
    {
        // in
        std::vector<Point> infrom, into;
        std::vector<float> infocus;
        std::vector<float> inopen;
        std::vector<float> inclose;
        for(auto& camera : cameras)
        {
            infrom.push_back(camera.position());
            into.push_back(camera.r2w(Point(w/2,h/2,0)));
            infocus.push_back(camera.z_focus);
            inopen.push_back(camera.open);
            inclose.push_back(camera.close);
        }

        // out
        std::vector<Point> outfrom(viewers), outto(viewers);
        std::vector<float> outfocus(viewers);
        std::vector<float> outopen(viewers);
        std::vector<float> outclose(viewers);
        float step = 1.f / float(viewers - 1);
        for(int i = 0; i < viewers; i++)
        {
            float t = float(i) * step;
            outfrom[i] = bezier(infrom.begin(), infrom.end()-1, t);
            outto[i] = bezier(into.begin(), into.end()-1, t);
            outfocus[i] = bezier(infocus.begin(), infocus.end()-1, t);
            outopen[i] = bezier(inopen.begin(), inopen.end()-1, t);
            outclose[i] = bezier(inclose.begin(), inclose.end()-1, t);
        }

        // new camera setup    
        cameras.clear();
        for(int i = 0; i < viewers; i++)
        {
            if(!use_camera(i, only))
                continue;
            
            // new camera
            cameras.push_back(Camera( outfrom[i], outto[i], w, h, fov ));
            // float open = time ? std::max(0.f, (float(i) - overlap) * step) : 0.f;
            // float close = time ? std::min(1.f, (float(i) + overlap) * step) : 0.f;

            // modele 50 mm, f/2
            // float radius = 0.025;  
            // float radius = 0.0125;  
            float focus = outfocus[i];
            // float focal = 0.050;
            // float focal = 0.1;
            // float film = (focus * focal) / (focus - focal);
            // float film = 0.050;

            // cameras.back().open = open;
            // cameras.back().close = close;
            // cameras.back().open = outopen[i];
            // cameras.back().close = outclose[i];
            cameras.back().open = 0;
            cameras.back().close = 0;
            cameras.back().lens = Lens(radius);
            cameras.back().z_focus = outfocus[i];
            // cameras.back().z_film = film;
            // cameras.back().z_focal = focal;
            // cameras.back().init();
        }
    }
}

void interpolate_line(std::vector<Camera>& cameras, int viewers, const std::vector<int>& only)
{
    int w = cameras[0].width, h = cameras[0].height;
    float radius = cameras[0].lens.radius();
    float fov = cameras[0].fov;

    for(auto& camera : cameras)
    {
        printf("%f %f %d %d\n", camera.width, camera.height, w, h);
        assert(camera.width == w && camera.height == h);
    }

    // interpolate cameras if needed
    if( viewers )
    {
        Point afrom =   cameras[0].position();
        Point bfrom =   cameras[1].position();
        Point ato =     cameras[0].r2w(Point(w/2,h/2,0));
        Point bto =     cameras[1].r2w(Point(w/2,h/2,0));
        float afocus =  cameras[0].z_focus;
        float bfocus =  cameras[1].z_focus;
        float aopen =   cameras[0].open;
        float bopen =   cameras[1].open;
        float aclose =  cameras[0].close;
        float bclose =  cameras[1].close;

        // new camera setup    
        cameras.clear();
        float step = 1.f / float(viewers - 1);
        for(int i = 0; i < viewers; i++)
        {
            if(!use_camera(i, only))
                continue;

            // new camera
            float t = float(i) * step;
            Point from = lerp(afrom, bfrom, t);
            Point to  = lerp(ato, bto, t);
            cameras.push_back(Camera( from, to, w, h, fov ));
            cameras.back().open = lerp(aopen, bopen, t);
            cameras.back().close = lerp(aclose, bclose, t);
            cameras.back().lens = Lens(radius);
            cameras.back().z_focus = lerp(afocus, bfocus, t);
        }
    }
}

void interpolate_grid(std::vector<Camera>& cameras, int x, int y, int viewers, const std::vector<int>& only)
{
    int w = cameras[0].width, h = cameras[0].height;
    float radius = cameras[0].lens.radius();
    float fov = cameras[0].fov;

    for(auto& camera : cameras)
    {
        printf("%f %f %d %d\n", camera.width, camera.height, w, h);
        assert(camera.width == w && camera.height == h);
    }

    // interpolate cameras if needed
    if( viewers )
    {
        // get camera origins
        Point afrom =   cameras[0].position();
        Point bfrom =   cameras[1].position();
        
        // camera from basis
        Vector up = normalize(cameras[0].up());
        Vector diag = bfrom - afrom;
        Vector dir = normalize(cross(diag, up));
        Vector right = normalize(cross(up, dir)); 

        // x length and y length
        float dx = std::abs(dot(diag, up));
        float dy = std::abs(dot(diag, right));

        float focus =  cameras[0].z_focus;
        float open =   cameras[0].open;
        float close =  cameras[0].close;

        // new camera setup    
        cameras.clear();
        float stepx = dx / float(x - 1);
        float stepy = dy / float(y - 1);
        int id = 0;
        for(int i = 0; i < y; i++)
        for(int j = 0; j < x; j++)
        {
            if(!use_camera(id++, only))
                continue;

            // new camera parameters
            float tx = float(j) * stepx;
            float ty = float(i) * stepy;

            // new camera position
            Point from = afrom + tx * right + ty * up;
            Point to = from + dir;
            cameras.push_back(Camera( from, to, w, h, fov ));
            cameras.back().open = open;
            cameras.back().close = close;
            cameras.back().lens = Lens(radius);
            cameras.back().z_focus = focus;
        }
    }
}

void interpolate_time(std::vector<Camera>& cameras, float overlap)
{
    float step = 1.f / float(cameras.size() - 1);
    for(unsigned int i = 0; i < cameras.size(); i++)
    {
        cameras[i].open = std::max(0.f, (float(i) - overlap) * step);
        cameras[i].close = std::min(1.f, (float(i) + overlap) * step);
    }
}

Camera read_camera(std::ifstream& in)
{
    Camera c;
    Point from, to;
    Vector up;
    printf("reading camera\n");
    std::string line;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;
        // parse line 
        std::istringstream iss(line);
        std::string header; 
        iss >> header;
        if( header == "res" )
            iss >> c.width >> c.height;                        
        else if( header == "fov" )
            iss >> c.fov;                                   
        else if( header == "from" )
            iss >> from.x >> from.y >> from.z;                        
        else if( header == "to" )
            iss >> to.x >> to.y >> to.z;                        
        else if( header == "up" )
            iss >> up.x >> up.y >> up.z;   
        else if( header == "z_far" )
            iss >> c.z_far;                  
        else if( header == "z_near" )
            iss >> c.z_near;                  
        else if( header == "z_film" )
            iss >> c.z_film;                  
        else if( header == "z_focus" )
            iss >> c.z_focus;                  
        else if( header == "radius" )
        {
            float radius;
            iss >> radius;
            c.lens = Lens(radius);                    
        }
        else if( header == "shutter" )
            iss >> c.open >> c.close;
    }

    c.film = Film(c.width, c.height); 
    c.tr_view = Lookat(from, to, up);
    c.tr_proj = Perspective(c.fov, float(c.width) / float(c.height), c.z_near, c.z_far);
    c.tr_film = Viewport(c.width, c.height); 
    c.init();
    return c;
}

void read_cameras(std::ifstream& in, std::vector<Camera>& cameras, const std::string& path)
{
    std::string line;
    int viewers = 0;
    bool time = false;
    float overlap = 1.5f;
    std::vector<int> only;

    bool bezier = false;
    bool cameraline = false;
    bool cameragrid = false;
    int x, y;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;

        std::istringstream iss(line);
        std::string header;
        iss >> header;
        if( header == "lerp" )
            // number of cameras to interpolate
            iss >> viewers;
        else if( header == "bezier" )
        {
            // number of cameras to interpolate
            iss >> viewers;
            bezier = true;
        }
        else if( header == "line" )
        {
            // number of cameras to interpolate
            iss >> viewers;
            cameraline = true;
        }
        else if( header == "grid" )
        {
            // number of cameras to interpolate
            iss >> x >> y;
            viewers = x*y;
            cameragrid = true;
        }
        else if( header == "time" )
            // time interpolation
            iss >> time;
        else if( header == "overlap" )
            // overlapping for motion blur
            iss >> overlap;
        else if( header == "only" )
        {
            // a subset of the cameras
            int c = -1;
            while( iss >> c )
            {
                only.push_back(c);
            }
        }
        else if( header == "newcamera" )
        {
            cameras.push_back(read_camera(in));
            // cameras.back().init();
            cameras.back().print();
        }
        else
            // new camera
            cameras.push_back(Camera(path+header));
    }

    if( cameraline )
    {
        assert(cameras.size() == 2);
        printf("interpolate line %d\n", viewers);
        interpolate_line(cameras, viewers, only);
    }
    else if( cameragrid )
    {
        assert(cameras.size() == 2);
        printf("interpolate grid (%d,%d) %d\n", x, y, viewers);
        interpolate_grid(cameras, x, y, viewers, only);
    }
    else if( bezier || viewers ) 
    {
        printf("interpolate beziers %d\n", viewers);
        interpolate_bezier(cameras, viewers, only);
    }

    if( !cameraline && !cameragrid && !viewers && !only.empty() )
    {
        std::vector<Camera> tmp;
        printf("subset %lu\n", only.size());
        for(auto index : only)
        {
            printf("%d ", index);
            tmp.push_back(Camera(cameras[index]));
        }
        printf("\n");
        cameras = std::move(tmp);
    }

    if( time )
    {
        printf("interpolate time \n");
        interpolate_time(cameras, overlap);
    }

    for( auto& camera : cameras ) camera.print();

    printf("cameras:\t%d\n", (int)cameras.size());
}

Material read_material(std::ifstream& in, const std::string& material_name, const std::string& path)
{
    Material m;
    m.name = material_name;
    // parse scene file
    std::string line;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;
        
        // parse line 
        std::istringstream iss(line);
        std::string header;
        iss >> header;
        if( header == "Kd" )
            iss >> m.diffuse.r >> m.diffuse.g >> m.diffuse.b;
        else if( header == "Ks" )
            iss >> m.specular.r >> m.specular.g >> m.specular.b;
        else if( header == "Ke" )
            iss >> m.emission.r >> m.emission.g >> m.emission.b;
        else if( header == "Ns" )
            iss >> m.ns;
        else if( header == "Ni" )
            iss >> m.ni;
        else if( header == "illum" )
            iss >> m.illum;
        else if( header == "scale" )
            iss >> m.texture_scale;
        else if( header == "sampling" )
        {
            iss >> header;
            if( header == "bicubic" )
                m.texture_sampling = 2;
            else if( header == "bilinear" )
                m.texture_sampling = 1;
            else
                m.texture_sampling = 0;
        }
        else if( header == "map_Kd" )
        {
            iss >> m.diffuse_filename;
            m.diffuse_filename = path + m.diffuse_filename;
        }
        else if( header == "map_Ks" )
        {
            iss >> m.specular_filename;
            m.specular_filename = path + m.specular_filename;
        }
        else if( header == "map_Ke" )
        {
            iss >> m.emission_filename;
            m.emission_filename = path + m.emission_filename;
        }
        else if( header == "map_Ns" )
        {
            iss >> m.ns_filename;
            m.ns_filename = path + m.ns_filename;
        }
        else if( header == "model" )
        {
            std::string model;
            iss >> model;
            // illumination models
            if( model == "plastic" )
                m.illum = 2; // plastic
            else if( model == "conductor" )
                m.illum = 5; // conductor
            else if( model == "dielectric" )
                m.illum = 7; // dielectric
            else if( model == "volume" )
                m.illum = 11; // volumes / hack 
            else if( model == "thin" )
                m.illum = 12; // thin_dielectric
            else if( model == "diffuse" )
                m.illum = 1; // 0-1 and others : diffuse (Lambert)
            else
                m.illum = 0; // 0-1 and others : diffuse (Lambert)
        }
        else
        {
            // end parsing the material section, putback characters and resume parsing
            for(int i= 0; i < int(line.size()); i++)
                in.unget();
            break;
        }
    }
    
    return m;
}

void read_geometry(std::ifstream& in, Builder& builder, const std::string& geometry_name, const std::string& path)
{
    // data
    int compact = 0, textures = 0;
    Geometry g_static, g_dynamic;
    Transform t = Identity();
    Material m;
    Point a, b, c, d;
    bool isBox = false;
    bool isQuad = false;
    bool newcache = false;

    printf("reading geometry %s\n", geometry_name.c_str());
    
    // get geometry filenames
    std::vector<std::string> filenames;
    std::string line;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;

        std::istringstream iss(line);
        std::string header;
        iss >> header;
        if( header == "compact" )
            iss >> compact;
        else if( header == "textures" )
            iss >> textures;
        else if( header == "newcache" )
            iss >> newcache;
        else if( header == "box" )
        {
            printf("reading box\n");
            Point pmin, pmax;
            iss >> pmin.x >> pmin.y >> pmin.z >> pmax.x >> pmax.y >> pmax.z;
            Vector scale = pmax - pmin;
            Vector mid((pmin + pmax)*0.5f);
            t = Translation(mid) * Scale(scale.x, scale.y, scale.z);
            isBox = true;
        }
        else if( header == "quad" )
        {
            printf("reading quad\n");
            iss >> a.x >> a.y >> a.z >> b.x >> b.y >> b.z >> c.x >> c.y >> c.z >> d.x >> d.y >> d.z;
            isQuad = true;
        }
        else if( header == "matrix" )
        {
            iss >> t;
        }
        else if( header == "newmaterial" )
        {
            std::string material_name;
            iss >> material_name;
            printf("reading material %s\n", material_name.c_str());
            m = read_material(in, material_name, path);
        }
        else
        {
            printf("reading obj file %s\n", (path+header).c_str());
            filenames.push_back(path + header);
        }
    }

    // dynamic geometry
    if( filenames.size() > 1 )
    {
        std::string static_cache = path + geometry_name + ".static";
        std::string dynamic_cache = path + geometry_name + ".dynamic";

        Geometry g_static, g_dynamic;

        if( !newcache && read_geometry_cache(static_cache, g_static) && read_geometry_cache(dynamic_cache, g_dynamic) )
        {
            // nothing to do
        }
        else
        {
            // keyframes
            std::vector<MeshData> data(filenames.size());
            #pragma omp parallel for
            for(int i = 0; i < int(filenames.size()); ++i)
                data[i] = read_mesh_data( filenames[i].c_str() );

            // split & compact geometry
            split_mesh( data, g_static, g_dynamic );

            // write cache if needed
            write_geometry_cache(g_static, static_cache);
            write_geometry_cache(g_dynamic, dynamic_cache);
        }

        // read textures
        if( textures )
        {
            g_static.textures = read_textures( g_static.materials );
            g_dynamic.textures = read_textures( g_dynamic.materials );
        }

        // add geometry
        builder.add(g_static);
        builder.add(g_dynamic);
    }
    // static geometry
    else if( filenames.size() == 1 )
    {
        std::string static_cache = path + geometry_name + ".static";

        Geometry g_static;

        if( !newcache && read_geometry_cache(static_cache, g_static) )
        {
            // nothing to do
        }
        else
        {
            // read geometry
            MeshData data = read_mesh_data( filenames[0].c_str());
            g_static = Geometry(data);

            // write cache if needed
            write_geometry_cache(g_static, static_cache);
        }

        // read textures
        if( textures )
            g_static.textures = read_textures( g_static.materials );

        // add geometry
        builder.add(g_static);
    }
    // box geometry specified
    else if( isBox )
    {   
        Geometry cube = make_cube(t);
        cube.materials[0] = m;
        cube.print();
        if( textures )
            cube.textures = read_textures( cube.materials );
        builder.add(cube);
    }
    else if( isQuad )
    {
        Geometry quad = make_quad(a, b, c, d);
        quad.materials[0] = m;
        quad.print();
        if( textures )
            quad.textures = read_textures( quad.materials );
        builder.add(quad);
    }
}

void read_params(std::ifstream& in, IntegratorConfig& oConfig)
{
    // parse file
    std::string line;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;

        // parse line 
        std::istringstream iss(line);
        std::string header;
        iss >> header;
        if( header == "algorithm" )
        {
            std::string acronym;
            iss >> acronym;
            oConfig.mAlgorithm = oConfig.GetAlgorithm(acronym);

            if( oConfig.mAlgorithm == IntegratorConfig::kAlgorithmMax ){
                printf("unknow algorithm '%s'\n", acronym.c_str());
                exit(1);                   
            }
        }
        else if( header == "adaptive" )
            iss >> oConfig.mUseAdaptive;
        else if( header == "max_samples" )
            iss >> oConfig.mAdaptiveMaxSamples;
        else if( header == "min_samples" )
            iss >> oConfig.mAdaptiveMinSamples;
        else if( header == "sample_limit" )
            iss >> oConfig.mSampleLimit;
        else if( header == "time_limit" )
            iss >> oConfig.mTimeLimit;
        else if( header == "reuse_limit" )
            oConfig.mReuseLimit = true;
        else if( header == "save_samples" )
            oConfig.mSaveSamples = true;
        else if( header == "save_natives" )
            oConfig.mSaveNatives = true;
        else if( header == "save_png" )
            iss >> oConfig.mSavePng;
        else if( header == "max_depth" )
            iss >> oConfig.mMaxPathLength;
        else if( header == "min_depth" )
            iss >> oConfig.mMinPathLength;
        else if( header == "output" )
            iss >> oConfig.mOutputName;
        else if( header == "viewer" )
            iss >> oConfig.mViewer;
        else if( header == "layers" )
            iss >> oConfig.mLayers;
        else if( header == "partial" )
            iss >> oConfig.mSavePartial;
        else if( header == "partial_times" )
        {
            int value;
            while(iss)
            {
                iss >> value;
                oConfig.mPartialTimes.insert(value);
            }
        }
        else if( header == "partial_samples" )
        {
            int value;
            while(iss)
            {
                iss >> value;
                oConfig.mPartialSamples.insert(value);
            }
        }
        else if( header == "save" )
            iss >> oConfig.mSave;
        else if( header == "mvpt_use_selection" )
            iss >> oConfig.mUseSelection;
        else if( header == "mvpt_use_selection_probability" )
            iss >> oConfig.mUseSelectionProbability;
        else if( header == "mvpt_use_mixture" )
            iss >> oConfig.mUseMixture;
    }
}

// # volume descritpion
// newvolume volumename
//     sigmaa X X X             # absorption
//     sigmas X X X             # scattering 
//     phaseg X                 # Henyey-Greenstein parameter (default 0 - optional)
//     inside materialname1
//     inside materialname2
//     inside materialname3
//     outside materialname4
//     outside materialname5
//     # if we want to enable / disable the BSDF interface for mediums enclosed by geometry
//     use_bsdf 0 | 1          # (default 0 - optional)
//     # if we want heterogeneous noise
//     noise value | simplex | perlin | cellular    
//     noisefreq X              # (default 10 - optional)
//     noiseseed X              # (default 1234 - optional)
//     animated 0/1
//     # if we want heterogeneous NanoVDB grid
//     nanovdb nanovdbfile 
//     box X X X X X X
//     material materialname    # optionnal to add an interface to the volume
//     
//     # transform world points in density function calls
//     matrix X X X X X X X X X X X X X X X X
//
// 1. if any "inside" field is specified and if the volume is not a 3D grid or NanoVBD grid then
// the volume is considered everywhere in the scene and outside all materials and cameras
// 2. if none of the "noise", or "nanovbd" fields is specified then
// the volume is considered constant (homogeneous)
// 3. if "nanovbd" field is specified then 
// the volume need a bounding box specified in the scene (for the intersector) using field "box", 
// or a transformation matrix with the field matrix
// May be also specified for non grid volumes to put them in a box, but not for now... 
// WARNING : the volume have to be declared after the geometry it refers to

void read_volume(std::ifstream& in, Builder& builder, const std::string& volume_name, const std::string& path)
{
    printf("reading volume %s\n", volume_name.c_str());
    
    // medium data
    MediumData mdata;
    mdata.volume_name = volume_name;
    // parse scene file
    std::string line;
    while(std::getline(in, line))
    {
        auto it = line.begin();
        // skip beginnin white spaces
        while( it != line.end() && std::isspace(*it) ) ++it;
        // skip comments
        if( *it == '#' ) continue;
        // exit at first empty line
        if( it == line.end() ) break;

        // parse line 
        std::istringstream iss(line);
        std::string header;
        iss >> header;
        if( header == "infinite" )
        {
            mdata.infinite = true;
        }
        if( header == "inside" )
        {
            std::string materialname;
            iss >> materialname;
            mdata.inside.push_back(materialname);
        }
        if( header == "outside" )
        {
            std::string materialname;
            iss >> materialname;
            mdata.outside.push_back(materialname);
        }
        else if( header == "bsdf" )
        {
            iss >> mdata.use_bsdf;
        }
        else if( header == "sigmaa" )
        {
            iss >> mdata.sigmaa.r >> mdata.sigmaa.g >> mdata.sigmaa.b;
        }
        else if( header == "sigmas" )
        {
            iss >> mdata.sigmas.r >> mdata.sigmas.g >> mdata.sigmas.b;
        }
        else if( header == "phaseg" )
        {
            iss >> mdata.phaseg;
        }
        else if( header == "noise" )
        {
            mdata.noise = true;
            std::string s_noise_type;
            iss >> s_noise_type;
            if( s_noise_type == "simplex" )         mdata.noise_type = DensityFunctionFastNoise::NoiseType::Simplex;
            else if( s_noise_type == "perlin" )     mdata.noise_type = DensityFunctionFastNoise::NoiseType::Perlin;
            else if( s_noise_type == "cellular" )   mdata.noise_type = DensityFunctionFastNoise::NoiseType::Cellular;
            else                                    mdata.noise_type = DensityFunctionFastNoise::NoiseType::Value;
        }
        else if( header == "noisefreq" )
        {
            iss >> mdata.noise_freq;
        }
        else if( header == "noiseseed" )
        {
            iss >> mdata.noise_seed;
        }
        else if( header == "animated" )
        {
            mdata.anim = true;
        }
        else if( header == "nanovdb" )
        {
            iss >> mdata.nanovdb_file;
            mdata.nanovdb_file = path+mdata.nanovdb_file;
        }
        else if( header == "grid" )
        {
            iss >> mdata.nanovdb_grid;
            mdata.nanovdb_grid = path+mdata.nanovdb_grid;
        }
        else if( header == "box" )
        {
            Point pmin, pmax;
            iss >> pmin.x >> pmin.y >> pmin.z >> pmax.x >> pmax.y >> pmax.z;
            Vector scale = pmax - pmin;
            Vector mid((pmin + pmax)*0.5f);
            mdata.tr.push_back(Translation(mid) * Scale(scale.x, scale.y, scale.z));
        }
        else if( header == "matrix" )
        {
            Transform t;
            iss >> t;
            mdata.tr.push_back(t);
        }
        else if( header == "material" )
        {
            iss >> mdata.material_name;
        }
    }

    // attach to builder 
    builder.mediumData.push_back(mdata);
}

void read_scene(std::string filename, std::vector<Camera>& cameras, Builder& builder, IntegratorConfig& oConfig)
{
    std::string path, name;
    const size_t last_slash_index = filename.find_last_of("/");
    if(std::string::npos != last_slash_index)
    {
        path = filename.substr(0,last_slash_index+1);
        name = filename.substr(last_slash_index+1);
    }

    // std::cout << "path " << path << std::endl;
    // std::cout << "name " << name << std::endl;

    std::ifstream in(filename, std::ios::in);
    if( in )
    {
        std::cout << "reading scene " << name << std::endl;
        // parse file
        std::string line;
        while(std::getline(in, line))
        {
            auto it = line.begin();

            // skip beginnin white spaces
            while( it != line.end() && std::isspace(*it) ) ++it;

            // parse line 
            std::istringstream iss(line);
            std::string header;
            iss >> header;
            if( header == "newcameras" )
            {
                read_cameras(in, cameras, path);
            }
            else if( header == "newcamera" )
            {
                cameras.push_back(read_camera(in));
                cameras.back().init();
                cameras.back().print();
            }
            else if( header == "newgeometry" )
            {
                std::string geometry_name;
                iss >> geometry_name;
                read_geometry(in, builder, geometry_name, path);
            }
            else if( header == "newintegrator" )
            {
                read_params(in, oConfig);
            }
            else if( header == "newvolume" )
            {
                std::string volume_name;
                iss >> volume_name;
                read_volume(in, builder, volume_name, path);
            }
        }
        // build the scene
        builder.build();
    }
    else
    {
        std::cerr << "error reading scene..." << name << std::endl;
        exit(1);
    }
}


#endif
