#pragma once

#include <vector>
#include <map>

#include "renderer/texture/texture.h"
#include "renderer/bsdf/bsdf.h"

#include "image.h"
#include "image_io.h"

struct TextureLib
{
    std::vector<std::shared_ptr<Texture>> textures;
    std::shared_ptr<Texture> default_texture = std::make_shared<TextureConstant>(Red());
    int size( ) const { return int(textures.size()); }
};

static TextureLib read_textures( std::vector<Material>& materials )
{
    TextureLib lib;
    std::map<std::string, int> texture_map;

    for( auto& material : materials )
    {
        if(!material.diffuse_filename.empty())
        {
            auto inserter = texture_map.insert( std::make_pair(material.diffuse_filename, texture_map.size()) );
            if( inserter.second )
            {
                Image diffuse = read_image(material.diffuse_filename.c_str());
                if(diffuse.data().size())
                {
                    material.diffuse_texture = lib.textures.size();
                    lib.textures.push_back(std::make_shared<TextureImage>(diffuse, (Sampling)material.texture_sampling, material.texture_repeat, material.texture_scale, material.texture_scale));
                }
            }
            else
            {
                material.diffuse_texture = inserter.first->second;
            }
        }

        if(!material.specular_filename.empty())
        {
            auto inserter = texture_map.insert( std::make_pair(material.specular_filename, texture_map.size()) );
            if( inserter.second )
            {
                Image specular = read_image(material.specular_filename.c_str());
                if(specular.data().size())
                {
                    material.specular_texture = lib.textures.size();
                    lib.textures.push_back(std::make_shared<TextureImage>(specular, (Sampling)material.texture_sampling, material.texture_repeat, material.texture_scale, material.texture_scale));
                }
            }
            else
            {
                material.specular_texture = inserter.first->second;
            }
        }

        if(!material.emission_filename.empty())
        {
            auto inserter = texture_map.insert( std::make_pair(material.emission_filename, texture_map.size()) );
            if( inserter.second )
            {
                Image emission = read_image(material.emission_filename.c_str());
                if(emission.data().size())
                {
                    material.emission_texture = lib.textures.size();
                    lib.textures.push_back(std::make_shared<TextureImage>(emission, (Sampling)material.texture_sampling, material.texture_repeat, material.texture_scale, material.texture_scale));
                }
            }
            else
            {
                material.emission_texture = inserter.first->second;
            }
        }

        if(!material.ns_filename.empty())
        {
            auto inserter = texture_map.insert( std::make_pair(material.ns_filename, texture_map.size()) );
            if( inserter.second )
            {
                Image ns = read_image(material.ns_filename.c_str());
                if(ns.data().size())
                {
                    material.ns_texture = lib.textures.size();
                    lib.textures.push_back(std::make_shared<TextureImage>(ns, (Sampling)material.texture_sampling, material.texture_repeat, material.texture_scale, material.texture_scale));
                }
            }
            else
            {
                material.ns_texture = inserter.first->second;
            }
        }
    }

    return lib;
}