#ifndef TIMER_H
#define TIMER_H

#include <chrono>

struct Timer
{
    typedef std::chrono::high_resolution_clock clock;
    typedef std::chrono::duration<int, std::nano> nanoseconds;
    typedef std::chrono::duration<int, std::micro> microseconds;
    typedef std::chrono::duration<int, std::milli> milliseconds;
    typedef std::chrono::duration<int, std::ratio<1>> seconds;
    typedef std::chrono::duration<int, std::ratio<60>> minutes;

    explicit Timer(bool run = true) {if (run) reset();}

    void reset() {start = clock::now();}

    nanoseconds nano() const {return std::chrono::duration_cast<nanoseconds>(clock::now() - start);}
    microseconds micro() const {return std::chrono::duration_cast<microseconds>(clock::now() - start);}
    seconds sec() const {return std::chrono::duration_cast<seconds>(clock::now() - start);}
    minutes min() const {return std::chrono::duration_cast<minutes>(clock::now() - start);}

    template <typename T, typename Traits>
    friend std::basic_ostream<T, Traits>& operator<<(std::basic_ostream<T, Traits>& out, const Timer& timer)
    {
        // return out << timer.sec().count() << " sec";
        return out << timer.micro().count() << " µs";
    }

    // template <typename T, typename Traits>
    // friend std::basic_ostream<T, Traits>& operator<<(std::basic_ostream<T, Traits>& out, const Timer& timer)
    // {
    //     auto duration = Timer::clock::now() - timer.start;

    //     auto m = std::chrono::duration_cast<minutes>(duration);        duration -= m;
    //     auto s = std::chrono::duration_cast<seconds>(duration);        duration -= s;
    //     auto ms = std::chrono::duration_cast<milliseconds>(duration);  duration -= ms;
    //     auto mus = std::chrono::duration_cast<microseconds>(duration); duration -= mus;
    //     auto ns = std::chrono::duration_cast<nanoseconds>(duration);

    //     return out << m.count() << " min " << s.count() << " sec "<< ms.count() << " ms "<< mus.count() << " µs "<< ns.count() << " ns ";
    // }

    clock::time_point start;
};

#endif