#pragma once

#include "vec.h"
#include <cmath>

//  Construction d'un repère ortho à partir d'un seul vecteur
//      "Generating a consistently oriented tangent space" 
//      http://people.compute.dtu.dk/jerf/papers/abstracts/onb.html
//  Correction pour les cas aberrants
//      "Building an Orthonormal Basis, Revisited"
//      http://jcgt.org/published/0006/01/01/


class Frame
{
public:

    Frame(){}
    Frame( const Vector& _n ) : n(_n) 
    {
        const float sign_z = std::copysign(1.0f, n.z);
        const float u = -1.0f / (sign_z + n.z);
        const float v = n.x * n.y * u;
        t = Vector(1.0f + sign_z * n.x * n.x * u, sign_z * v, -sign_z * n.x);
        b = Vector(v, sign_z + n.y * n.y * u, -n.y);
    }

    Frame( const Vector& _n, const Vector& _dpdu ) : n(_n), t(normalize(_dpdu)), b(cross(n, t)) {}

    // local frame to world transformation    
    Vector ToWorld( const Vector& local )  const
    {
        return local.x * t + local.y * b + local.z * n;
    }
    
    // world to local frame transformation    
    Vector ToLocal( const Vector& global ) const
    {
        return Vector(dot(global, t), dot(global, b), dot(global, n));
    }
    
private:
    // Base dans le repère local
    Vector n;
    Vector t;
    Vector b;
};
