
#include <algorithm>

#include "color.h"

float length(const Color& c)
{
    return std::sqrt(c.r * c.r + c.g * c.g + c.b * c.b);
}

Color abs(const Color& c) 
{
    return Color(std::abs(c.r), std::abs(c.g), std::abs(c.b), c.a);
}

Color exp(const Color& c)
{
    return Color(std::exp(c.r), std::exp(c.g), std::exp(c.b), c.a);
}

Color pow(const Color& c, float exp)
{
    return Color(std::pow(c.r, exp), std::pow(c.g, exp), std::pow(c.b, exp), c.a);
}

Color sqrt(const Color& c)
{
    return Color(std::sqrt(c.r), std::sqrt(c.g), std::sqrt(c.b), c.a);
}

Color log(const Color& c)
{
    return Color(std::log(c.r), std::log(c.g), std::log(c.b), c.a);
}

Color min(const Color& a, const Color& b)
{
    return Color(std::min(a.r, b.r), std::min(a.g, b.g), std::min(a.b, b.b), std::min(a.a, b.a));
}

Color max(const Color& a, const Color& b)
{
    return Color(std::max(a.r, b.r), std::max(a.g, b.g), std::max(a.b, b.b), std::max(a.a, b.a));
}

Color min(const Color& c, const float v)
{
    return Color(std::min(c.r, v), std::min(c.g, v), std::min(c.b, v), std::min(c.a, v));
}

Color max(const Color& c, const float v)
{
    return Color(std::max(c.r, v), std::max(c.g, v), std::max(c.b, v), std::max(c.a, v));
}

Color min(const float v, const Color& c)
{
    return Color(std::min(c.r, v), std::min(c.g, v), std::min(c.b, v), std::min(c.a, v));
}

Color max(const float v, const Color& c)
{
    return Color(std::max(c.r, v), std::max(c.g, v), std::max(c.b, v), std::max(c.a, v));
}

Color clamp(const Color& c, float vmin, float vmax)
{
    return min(vmax, max(c, vmin));    
}

Color Black( )
{
    return Color(0, 0, 0);
}

Color White( )
{
    return Color(1, 1, 1);
}

Color Red( )
{
    return Color(1, 0, 0);
}

Color Green( )
{
    return Color(0, 1, 0);
}

Color Blue( )
{
    return Color(0, 0, 1);
}

Color Yellow( )
{
    return Color(1, 1, 0);
}

Color operator+ ( const Color& a, const Color& b )
{
    return Color(a.r + b.r, a.g + b.g, a.b + b.b, a.a + b.a);
}

Color operator- ( const Color& c )
{
    return Color(-c.r, -c.g, -c.b, -c.a);
}

Color operator- ( const Color& a, const Color& b )
{
    return a + (-b);
}

Color operator* ( const Color& a, const Color& b )
{
    return Color(a.r * b.r, a.g * b.g, a.b * b.b, a.a * b.a);
}

Color operator* ( const float k, const Color& c )
{
    return Color(c.r * k, c.g * k, c.b * k, c.a * k);
}

Color operator* ( const Color& c, const float k )
{
    return k * c;
}

Color operator+ ( const float k, const Color& c )
{
    return Color(c.r + k, c.g + k, c.b + k, c.a + k);
}

Color operator+ ( const Color& c, const float k )
{
    return k + c;
}

Color operator/ ( const Color& a, const Color& b )
{
    return Color(a.r / b.r, a.g / b.g, a.b / b.b, a.a / b.a);
}

Color operator/ ( const float k, const Color& c )
{
    return Color(k / c.r, k / c.g, k / c.b, k / c.a);
}

Color operator/ ( const Color& c, const float k )
{
    float kk= 1 / k;
    return kk * c;
}
